import re
from enum import Enum
from urllib.parse import urlparse

from wandb.sdk.artifacts._validators import is_artifact_registry_project


def parse_s3_url_to_s3_uri(url) -> str:
    """Convert an S3 HTTP(S) URL to an S3 URI.

    Arguments:
        url (str): The S3 URL to convert, in the format
                   'http(s)://<bucket>.s3.<region>.amazonaws.com/<key>'.
                   or 'http(s)://<bucket>.s3.amazonaws.com/<key>'

    Returns:
        str: The corresponding S3 URI in the format 's3://<bucket>/<key>'.

    Raises:
        ValueError: If the provided URL is not a valid S3 URL.
    """
    # Regular expression to match S3 URL pattern
    s3_pattern = r"^https?://.*s3.*amazonaws\.com.*"
    parsed_url = urlparse(url)

    # Check if it's an S3 URL
    match = re.match(s3_pattern, parsed_url.geturl())
    if not match:
        raise ValueError("Invalid S3 URL")

    # Extract bucket name and key
    bucket_name, *_ = parsed_url.netloc.split(".")
    key = parsed_url.path.lstrip("/")

    # Construct the S3 URI
    s3_uri = f"s3://{bucket_name}/{key}"

    return s3_uri


class PathType(Enum):
    """We have lots of different paths users pass in to fetch artifacts, projects, etc.

    This enum is used for specifying what format the path is in given a string path.
    """

    PROJECT = "PROJECT"
    ARTIFACT = "ARTIFACT"


def parse_org_from_registry_path(path: str, path_type: PathType) -> str:
    """Parse the org from a registry path.

    Essentially fetching the "entity" from the path but for Registries the entity is actually the org.

    Args:
        path (str): The path to parse. Can be a project path <entity>/<project> or <project> or an
        artifact path like <entity>/<project>/<artifact> or <project>/<artifact> or <artifact>
        path_type (PathType): The type of path to parse.
    """
    parts = path.split("/")
    expected_parts = 3 if path_type == PathType.ARTIFACT else 2

    if len(parts) >= expected_parts:
        org, project = parts[:2]
        if is_artifact_registry_project(project):
            return org
    return ""



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from wandb.data_types import Table
from wandb.errors import Error


@dataclass
class VisualizeSpec:
    name: str
    key: str = ""

    @property
    def config_value(self) -> dict[str, Any]:
        return {
            "id": self.name,
            "historyFieldSettings": {"x-axis": "_step", "key": self.key},
        }

    @property
    def config_key(self) -> tuple[str, str, str]:
        return ("_wandb", "viz", self.key)


@dataclass
class Visualize:
    table: Table
    spec: VisualizeSpec

    def set_key(self, key: str) -> None:
        self.spec.key = key


def visualize(id: str, value: Table) -> Visualize:
    if not isinstance(value, Table):
        raise Error(
            f"Expected `value` to be `wandb.Table` type, instead got {type(value).__name__}"
        )
    return Visualize(table=value, spec=VisualizeSpec(name=id))



from __future__ import annotations

from typing import TYPE_CHECKING

from wandb.plot.custom_chart import plot_table

if TYPE_CHECKING:
    import wandb
    from wandb.plot.custom_chart import CustomChart


def bar(
    table: wandb.Table,
    label: str,
    value: str,
    title: str = "",
    split_table: bool = False,
) -> CustomChart:
    """Constructs a bar chart from a wandb.Table of data.

    Args:
        table (wandb.Table): A table containing the data for the bar chart.
        label (str): The name of the column to use for the labels of each bar.
        value (str): The name of the column to use for the values of each bar.
        title (str): The title of the bar chart.
        split_table (bool): Whether the table should be split into a separate section
            in the W&B UI. If `True`, the table will be displayed in a section named
            "Custom Chart Tables". Default is `False`.

    Returns:
        CustomChart: A custom chart object that can be logged to W&B. To log the
            chart, pass it to `wandb.log()`.

    Example:
        ```
        import random
        import wandb

        # Generate random data for the table
        data = [
            ['car', random.uniform(0, 1)],
            ['bus', random.uniform(0, 1)],
            ['road', random.uniform(0, 1)],
            ['person', random.uniform(0, 1)],
        ]

        # Create a table with the data
        table = wandb.Table(data=data, columns=["class", "accuracy"])

        # Initialize a W&B run and log the bar plot
        with wandb.init(project="bar_chart") as run:

            # Create a bar plot from the table
            bar_plot = wandb.plot.bar(
                table=table,
                label="class",
                value="accuracy",
                title="Object Classification Accuracy",
            )

            # Log the bar chart to W&B
            run.log({'bar_plot': bar_plot})
        ```
    """
    return plot_table(
        data_table=table,
        vega_spec_name="wandb/bar/v0",
        fields={"label": label, "value": value},
        string_fields={"title": title},
        split_table=split_table,
    )



"""Module containing the WBURLs class and WBURL dataclass.

Used to store predefined URLs that can be associated with a name. The URLs are
shortened using with the `wandb.me` domain, using dub.co as the shortening service.
If the URLs need to be updates, use the dub.co service to point to the new URL.
"""

from __future__ import annotations

from dataclasses import dataclass


@dataclass
class WBURL:
    url: str
    description: str


class Registry:
    """A collection of URLs that can be associated with a name."""

    def __init__(self):
        self.urls: dict[str, WBURL] = {
            "wandb-launch": WBURL(
                "https://wandb.me/launch",
                "Link to the W&B launch marketing page",
            ),
            "wandb-init": WBURL(
                "https://wandb.me/wandb-init",
                "Link to the wandb.init reference documentation page",
            ),
            "define-metric": WBURL(
                "https://wandb.me/define-metric",
                "Link to the W&B developer guide documentation page on wandb.define_metric",
            ),
            "developer-guide": WBURL(
                "https://wandb.me/developer-guide",
                "Link to the W&B developer guide top level page",
            ),
            "wandb-core": WBURL(
                "https://wandb.me/wandb-core",
                "Link to the documentation for the wandb-core service",
            ),
            "wandb-server": WBURL(
                "https://wandb.me/wandb-server",
                "Link to the documentation for the self-hosted W&B server",
            ),
            "multiprocess": WBURL(
                "https://wandb.me/multiprocess",
                (
                    "Link to the W&B developer guide documentation page on how to "
                    "use wandb in a multiprocess environment"
                ),
            ),
        }

    def url(self, name: str) -> str:
        """Get the URL associated with the given name."""
        wb_url = self.urls.get(name)
        if wb_url:
            return wb_url.url
        raise ValueError(f"URL not found for {name}")

    def description(self, name: str) -> str:
        """Get the description associated with the given name."""
        wb_url = self.urls.get(name)
        if wb_url:
            return wb_url.description
        raise ValueError(f"Description not found for {name}")


# This is an instance of the Links class that can be used to access the URLs
url_registry = Registry()
